SystemOrganization addCategory: #SWA014!

Object subclass: #SWA014Spawner
	instanceVariableNames: 'game playerBlob swaworld'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Spawner commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am the Spawner. I spawn all the things you see. I observe Enemy, to spawn a new Enemy, if it gets eaten.
!

!SWA014Spawner class methodsFor: 'initialization' stamp: 'MV 2/5/2017 15:09'!
newWith: aGame
	
	^ SWA014Spawner new
		initializeWith: aGame;
		yourself.
	
	
	! !

!SWA014Spawner methodsFor: 'constants' stamp: 'Tc 2/3/2017 11:26'!
defaultMaxEnemySize

	^ 100.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:39'!
game
	
	^ game.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:39'!
game: aGame
	
	game := aGame.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:43'!
giveUpgradesToEnemy: enemy
	|upgradeNumberToEnable listOfKeys randomUpgradeKey|

	listOfKeys := self game upgradeDictionary keys asOrderedCollection.
	upgradeNumberToEnable := listOfKeys size min: self playerBlob level.
	
	1 to: upgradeNumberToEnable do: [:aNumber|
		randomUpgradeKey := listOfKeys atRandom.
		self game enable: randomUpgradeKey for: enemy.
		listOfKeys remove: randomUpgradeKey].! !

!SWA014Spawner methodsFor: 'constants' stamp: 'MV 2/4/2017 14:31'!
initialNumberOfEnemies

	^ 50.! !

!SWA014Spawner methodsFor: 'initialization' stamp: 'MV 2/5/2017 12:50'!
initializeWith: aGame
	super initialize.
	
	self 
		game: aGame;
		spawnWorld;
		spawnPlayerBlob;
		spawnInitialEnemies;
		spawnUpgradeBar;
		openScoreWindow;
		yourself.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:43'!
openScoreWindow
	|newScore|
	
	newScore := SWA014Score new. 
	self game addMorph: newScore.
	self playerBlob addDependent: newScore.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:40'!
playerBlob

	^ playerBlob.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:41'!
playerBlob: aLivingThing

	playerBlob := aLivingThing.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 14:51'!
randomEnemySize
	|maxSize|
	
	"In a quater of all cases we spawn a realy big (as big as the playerBlob) guy"
	maxSize := (Random new isTrueIn: 1 outOf: 4) 
		ifTrue: [(self playerBlob bodysize) asInteger]
		ifFalse: [self defaultMaxEnemySize].
	
	^ Random new nextInt: maxSize.
! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 19:56'!
spawnEnemy
	|enemy random|
	
	random := Random new.
	enemy := SWA014Enemy new.
	
	"Im Gegensatz zum Spieler erhalten Enemies ihre Upgrades vom Spawner abhängig vom Level des Spielers und nicht durch Evolution."
	self giveUpgradesToEnemy: enemy.
	
	self playerBlob listOfLivingThings add: enemy.
	self swaworld addMorph: enemy.

	enemy
		addDependent: self;
		bodysize: self randomEnemySize;
		swaworld: self swaworld;
		realCenter: (random nextInt: self swaworld width)@(random nextInt: self swaworld height);
		listOfLivingThings: self playerBlob listOfLivingThings;
		yourself.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:50'!
spawnInitialEnemies

	1 to: self initialNumberOfEnemies do: [:aNumber| self spawnEnemy]. ! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 17:11'!
spawnPlayerBlob
	|newPlayerBlob|
	
	newPlayerBlob := SWA014PlayerBlob new.
	
	newPlayerBlob
		color: Color red;
		bodysize: 60;
		swaworld: self swaworld;
		game: self game;
		realCenter: self swaworld extent / 2;
		yourself.
	
	self swaworld addMorph: newPlayerBlob.
	self playerBlob: newPlayerBlob.
	self game playerBlob: newPlayerBlob.
	self playerBlob listOfLivingThings add: newPlayerBlob.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:55'!
spawnUpgradeBar
	|aBar|
	
	aBar := SWA014UpgradeBar new position: self upgradeBarPosition.
	
	self game
		addMorph: aBar;
		upgradeBar: aBar;
		yourself.
	! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/5/2017 15:37'!
spawnWorld
	
	self swaworld: SWA014World new.
	self game addMorph: self swaworld.
	self swaworld
		realCenter: self game center;
		yourself.
	self swaworld image center: self game center.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:49'!
swaworld

	^ swaworld.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:49'!
swaworld: aWorld

	swaworld := aWorld.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/5/2017 13:01'!
update: aMessage
	
	self spawnEnemy.! !

!SWA014Spawner methodsFor: 'constants' stamp: 'MV 2/4/2017 16:39'!
upgradeBarPosition

	^ (self game bounds right - 125) @ (self game bounds top + 50).! !

CircleMorph subclass: #SWA014LivingThing
	instanceVariableNames: 'bodysize speed speedSaver direction maxAllowedSize minAllowedSize minAllowedSpeed maxAllowedSpeed realCenter upgradeList game swaworld listOfLivingThings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014LivingThing commentStamp: 'MV 2/5/2017 17:10' prior: 0!
I am a LivingThing. I am the superclass of all moving objects participating in the game. I implement their logic for movement, collisions etc.!

SWA014LivingThing subclass: #SWA014Enemy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Enemy commentStamp: 'MV 2/5/2017 17:12' prior: 0!
I am an Enemy. I will try to kill you and other LivingThings. You could eat me to grow. :) I inherit from LivingThing, where most of my logic is described.!

!SWA014Enemy methodsFor: 'movement' stamp: 'TZ 2/2/2017 20:29'!
calculateDirection: target
	"Enemys should follow the nearest object, which could be eaten by them. Also they should run away, if there is a bigger LivingThing (Player or Enemy) nearby."
	|direction|
	
	direction := super calculateDirection: target realCenter.
	^ self < target
		ifTrue: [direction negated]
		ifFalse: [direction].
	! !

!SWA014Enemy methodsFor: 'collision handling' stamp: 'MV 2/5/2017 13:13'!
getEaten

	self changed: self. "The Spawner should spawn an Enemy as a replacement"
	self delete.! !

!SWA014Enemy methodsFor: 'override' stamp: 'TZ 2/2/2017 20:29'!
step

	self 
		direction: (self calculateDirection: self nearestLivingThing);
		moveAutomatically;
		yourself. 
	
	self upgradeList do: [:aUpgrade| aUpgrade activateIfNotCooling].
	
	super step.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
< aLivingThing

	^ self bodysize < aLivingThing bodysize.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
> aLivingThing

	^ self bodysize > aLivingThing bodysize.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
addMassOf: aLivingThing

	self bodysize: 2 * ((self squaredRadius + aLivingThing squaredRadius) sqrt).! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:50'!
bigNumber

	^ SmallInteger maxVal.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
bodysize

	^ bodysize.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'MV 2/5/2017 15:20'!
bodysize: aSize
	
	bodysize := self maxAllowedSize min: (self minAllowedSize max: aSize).
! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:17'!
calculateDirection: target

	^ self vectorToPoint: target.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/5/2017 15:25'!
calculateRealCenterFor: aPosition
	"Converts a absolute position to a relative position like realCenter"
	
	^ aPosition - self swaworld realCenter + self swaworld radius asPoint.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/5/2017 15:22'!
checkForBorderCollisions
	 
		self realCenter: (self realCenter x min: (self swaworld width - self radius))@(self realCenter y min: (self swaworld height - self radius)).
		self realCenter: (self realCenter x max: self radius)@(self realCenter y max: self radius).! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/4/2017 18:56'!
checkForCollisions

	self checkForCollisionsWithLivingThings.
	self checkForBorderCollisions.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/4/2017 17:19'!
checkForCollisionsWithLivingThings

	self listOfLivingThings do: [:aLivingThing|
		(self checkForOverlapWith: aLivingThing) ifTrue: [
			aLivingThing < self 
				ifTrue: [
					aLivingThing goBehind.
					(self squaredDistanceTo: aLivingThing realCenter) < self squaredRadius ifTrue: [
						self eat: aLivingThing]]
				ifFalse: [aLivingThing comeToFront].
			self swaworld image goBehind]] without: self.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'TZ 2/2/2017 19:29'!
checkForOverlapWith: enemy

	^ self bottom > enemy top and: 
	    self top < enemy bottom and:
	    self left < enemy right and:
	    self right > enemy left.
! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:31'!
direction

	^ direction.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:32'!
direction: aDirection

	direction := aDirection.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/5/2017 15:33'!
eat: aLivingThing

	self addMassOf: aLivingThing.
	self listOfLivingThings remove: aLivingThing.

	aLivingThing getEaten.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'TZ 2/2/2017 19:46'!
getEaten
	
	self delete.! !

!SWA014LivingThing methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:00'!
initialize
	super initialize.
	
	self 
		maxAllowedSize: 700;
		maxAllowedSpeed: 100;
		minAllowedSize: 10;
		minAllowedSpeed: 0;
		speed: 2.0;
		speedSaver: 2.0;
		direction: 0.0 asPoint;
		borderWidth: 2; "these initial values don't change at runtime"
		upgradeList: Dictionary new;
		listOfLivingThings: Set new;
		yourself.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'MV 2/4/2017 15:41'!
listOfLivingThings
	
	^ listOfLivingThings.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'MV 2/4/2017 15:42'!
listOfLivingThings: aList
	
	listOfLivingThings := aList.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:50'!
maxAllowedSize

	^ maxAllowedSize.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:50'!
maxAllowedSize: aNumber

	maxAllowedSize := aNumber.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:51'!
maxAllowedSpeed

	^ maxAllowedSpeed.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:51'!
maxAllowedSpeed: aNumber

	maxAllowedSpeed := aNumber.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:51'!
minAllowedSize

	^ minAllowedSize.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:51'!
minAllowedSize: aNumber

	minAllowedSize := aNumber.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:51'!
minAllowedSpeed

	^ minAllowedSpeed.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:51'!
minAllowedSpeed: aNumber

	minAllowedSpeed := aNumber.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/3/2017 13:25'!
moveAutomatically

	self realCenter: self realCenter + (self speed * self direction).! !

!SWA014LivingThing methodsFor: 'vision' stamp: 'MV 2/4/2017 17:21'!
nearestLivingThing
	|nearestEnemyobject smallestDistance currentDistance |

	smallestDistance := self bigNumber.
	nearestEnemyobject := self.
		
	self listOfLivingThings do: [:livingThing| 
		currentDistance := self squaredDistanceTo: (livingThing realCenter).
		currentDistance  < smallestDistance ifTrue: [
			smallestDistance := currentDistance.
			nearestEnemyobject := livingThing]]
	without: self.

	^ nearestEnemyobject.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:43'!
radius

	^ self bodysize / 2.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:42'!
realCenter

	^ realCenter.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/3/2017 13:28'!
realCenter: aPoint

	realCenter := aPoint.
	self refreshCenter.! !

!SWA014LivingThing methodsFor: 'upgrade management' stamp: 'TZ 1/27/2017 19:23'!
receiveUpgrade: anUpgrade

	self addMorph: anUpgrade.
	anUpgrade hide.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:53'!
refreshCenter

	self center: self swaworld realCenter - (self swaworld extent / 2) + self realCenter.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:44'!
refreshSize

	self extent: self bodysize asPoint.
	
	! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:53'!
saveSpeed

	self speedSaver: self speed.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:53'!
speed

	^ speed.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/5/2017 15:21'!
speed: aSpeed

	speed := self maxAllowedSpeed min: (self minAllowedSpeed max: aSpeed).
! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:55'!
speedSaver

	^ speedSaver.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:55'!
speedSaver: aSpeed

	speedSaver := aSpeed.! !

!SWA014LivingThing methodsFor: 'vision' stamp: 'TZ 2/2/2017 19:33'!
squaredDistanceTo: enemy
	"Pythagoras' theorem without sqrt"
	
	^ (enemy x - self realCenter x) squared + (enemy y - self realCenter y) squared. ! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:34'!
squaredRadius
	"Der Radius wird Quadriert, damit wir, wenn wir mit ihm vergleichen, keine Wurzel ziehen müssen (Phytagoras)"
	
	^ self radius squared. ! !

!SWA014LivingThing methodsFor: 'override' stamp: 'TZ 2/2/2017 19:35'!
step

	self
		checkForCollisions;
		refreshSize;
		yourself.! !

!SWA014LivingThing methodsFor: 'override' stamp: 'TZ 2/2/2017 19:56'!
stepTime

	^ 10.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:42'!
swaworld

	^ swaworld.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:43'!
swaworld: aWorld
	
	swaworld := aWorld.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:43'!
upgradeList

	^ upgradeList.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:43'!
upgradeList: aList

	upgradeList := aList.! !

!SWA014LivingThing methodsFor: 'vision' stamp: 'MV 2/4/2017 18:37'!
vectorToPoint: aPoint
	
	^ (aPoint - self realCenter) normalizedWithoutZeroDivide.! !

SWA014LivingThing subclass: #SWA014PlayerBlob
	instanceVariableNames: 'level'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014PlayerBlob commentStamp: 'MV 2/5/2017 17:16' prior: 0!
I am the PlayerBlob. I will follow your orders and with me you can play the game. I eat Enemies until I evolve, so you can get Upgrades.!

!SWA014PlayerBlob methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:01'!
checkForEvolution

	self bodysize > self evolutionSize ifTrue: [self evolve].! !

!SWA014PlayerBlob methodsFor: 'collision handling' stamp: 'MV 2/5/2017 13:13'!
eat: aLivingThing
	super eat: aLivingThing.
	
	self
		changed: self; "The Score should increase"
		checkForEvolution;
		yourself.! !

!SWA014PlayerBlob methodsFor: 'constants' stamp: 'MV 2/5/2017 16:00'!
evolutionSize
	
	^ 600.! !

!SWA014PlayerBlob methodsFor: 'body transformation' stamp: 'MV 2/5/2017 15:48'!
evolve

	self game showEvolveOptions.
	
	self
		resetAllSizes;
		level: self level + 1;
		yourself.
	! !

!SWA014PlayerBlob methodsFor: 'accessing' stamp: 'TZ 1/27/2017 18:22'!
game
	
	^ game.! !

!SWA014PlayerBlob methodsFor: 'accessing' stamp: 'TZ 1/27/2017 18:22'!
game: aGame

	game := aGame.! !

!SWA014PlayerBlob methodsFor: 'collision handling' stamp: 'TZ 2/2/2017 19:48'!
getEaten

	self game gameOver.! !

!SWA014PlayerBlob methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:43'!
initialize
	super initialize.
	
	self
		speed: 3;
		level: 0;
		yourself.! !

!SWA014PlayerBlob methodsFor: 'accessing' stamp: 'TZ 2/2/2017 18:59'!
level

	^ level.! !

!SWA014PlayerBlob methodsFor: 'accessing' stamp: 'TZ 2/2/2017 18:59'!
level: aNumber

	level := aNumber.! !

!SWA014PlayerBlob methodsFor: 'movement' stamp: 'TZ 2/3/2017 13:26'!
moveAutomatically

	self swaworld realCenter: self swaworld realCenter - (self speed * self direction).
	self realCenter: self realCenter + (self speed * self direction).! !

!SWA014PlayerBlob methodsFor: 'upgrade management' stamp: 'TZ 1/27/2017 18:21'!
receiveUpgrade: anUpgrade

	self game upgradeBar add: anUpgrade.! !

!SWA014PlayerBlob methodsFor: 'movement' stamp: 'MV 2/4/2017 17:08'!
refreshCenter

	self swaworld realCenter: self swaworld realCenter + ((self calculateRealCenterFor: self game center) - self realCenter).
	super refreshCenter.! !

!SWA014PlayerBlob methodsFor: 'body transformation' stamp: 'MV 2/5/2017 15:55'!
resetAllSizes

	self listOfLivingThings do: [:aLivingThing| aLivingThing bodysize: aLivingThing bodysize / self shrinkFactor].! !

!SWA014PlayerBlob methodsFor: 'constants' stamp: 'MV 2/5/2017 15:55'!
shrinkFactor
	
	^ 10.! !

!SWA014PlayerBlob methodsFor: 'override' stamp: 'MV 1/21/2017 15:30'!
step
	super step.
	
	self 
		direction: (self calculateDirection: (self calculateRealCenterFor: World activeHand center));
		moveAutomatically;
		yourself.! !

Morph subclass: #SWA014EvolveMenu
	instanceVariableNames: 'game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014EvolveMenu commentStamp: 'MV 2/5/2017 21:08' prior: 0!
I am the EvolveMenu. I will be shown, if you gain enouth points. I will spawn a few EvolveOptions, where you can choose your upgrade. 
!

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'TZ 1/27/2017 18:38'!
game
	
	^ game.! !

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'TZ 1/27/2017 18:38'!
game: aGame

	game := aGame.! !

!SWA014EvolveMenu methodsFor: 'initialization' stamp: 'Tc 2/3/2017 11:25'!
initialize
	super initialize.
	
	self color: Color transparent.! !

!SWA014EvolveMenu methodsFor: 'constants' stamp: 'MV 2/5/2017 14:35'!
maximumNumberOfOptions

	^ 3.! !

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'MV 2/5/2017 15:04'!
positionFor: anOption from: anOptionList

	^ ((anOptionList indexOf: anOption) / (anOptionList size  + 1) * self width) @ (self center y).! !

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'MV 2/5/2017 15:12'!
showOptions: anOptionList
	|evolveOption|
	
	"Wir schmeißen so lange Elemente aus der Liste, bis wir maximal maximumNumberOfOptions in ihr übrig haben"
	1 to: (anOptionList size - self maximumNumberOfOptions) do: [:aNumber| anOptionList remove: anOptionList atRandom].
	
	anOptionList do: [:anOption|
		evolveOption := SWA014EvolveOption newWithUpgrade: anOption.
		evolveOption 
			game: self game;
			center: (self positionFor: anOption from: anOptionList);
			yourself.
		self addMorph: evolveOption].
	
	anOptionList isEmpty ifFalse: [self game pause].! !

Morph subclass: #SWA014EvolveOption
	instanceVariableNames: 'upgradeKey game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014EvolveOption commentStamp: 'MV 2/5/2017 21:08' prior: 0!
I am an EvolveOption. I am one of the options, you are shown by the EvolveMenu. Here you can get the upgrade I display by clicking on the ChooseButton, which is created by me.
!

!SWA014EvolveOption class methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:47'!
newWithUpgrade: anUpgradeKey

	^ self new
		initializeWith: anUpgradeKey;
		yourself.! !

!SWA014EvolveOption methodsFor: 'initialization' stamp: 'TZ 1/27/2017 16:24'!
createChooseButton
	| chooseButton |
	
	chooseButton := SWA014ChooseButton new.
	chooseButton center: (self center x)@(self height * 3 / 4).
	self addMorph: chooseButton.! !

!SWA014EvolveOption methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:45'!
createStringMorphForOption
	| stringMorph |

	stringMorph := StringMorph new.
	stringMorph acceptValue: self upgradeKey.	
	stringMorph center: (self center x) @ (self height * 1 / 4).
	self addMorph: stringMorph.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:30'!
game

	^ game.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:30'!
game: aGame

	game := aGame.! !

!SWA014EvolveOption methodsFor: 'upgrade managment' stamp: 'MV 2/5/2017 16:45'!
getChosen

	self game enable: self upgradeKey for: self game playerBlob.
	self owner delete.
	self game continue.
	self game upgradesUnavailableToPlayerBlob remove: self upgradeKey ifAbsent: [].! !

!SWA014EvolveOption methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:46'!
initializeWith: anUpgradeKey
	
	self 
		extent:  150 @ 250;
		color: Color white;
		upgradeKey: anUpgradeKey;
		createChooseButton;
		createStringMorphForOption;
		yourself.
! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:45'!
upgradeKey

	^ upgradeKey.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:45'!
upgradeKey: anUpgradeKey

	upgradeKey := anUpgradeKey.! !

Morph subclass: #SWA014Game
	instanceVariableNames: 'upgradeDictionary upgradesUnavailableToPlayerBlob playerBlob isPaused upgradeBar aPosition spawner'
	classVariableNames: 'Highscore'
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Game commentStamp: 'MV 2/5/2017 17:04' prior: 0!
I am the Game. I handle many events and react to them. For example I can enable an Upgrade for somebody or react if you press a Key on your Keyboard. At the begin of each game I will initialize some data structures.!

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/4/2017 17:18'!
continue

	self playerBlob listOfLivingThings do: [:aLivingThing| aLivingThing speed: aLivingThing speedSaver].
	self isPaused: false.! !

!SWA014Game methodsFor: 'upgrade managment' stamp: 'Tc 2/3/2017 12:03'!
enable: anUpgradeKey for: aLivingThing
	|anUpgrade|
	
	anUpgrade := (self upgradeDictionary at: anUpgradeKey ifAbsent: [SWA014NullUpgrade]) new.
	anUpgrade upgradeOwner: aLivingThing.
	aLivingThing receiveUpgrade: anUpgrade.
	aLivingThing upgradeList add: (anUpgradeKey->anUpgrade).
! !

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/4/2017 13:58'!
gameOver

	self delete.! !

!SWA014Game methodsFor: 'input events' stamp: 'MV 2/4/2017 15:17'!
handleKeystroke: anEvent
	|keyValue upgradeKey|
	
	
	
	keyValue := anEvent keyValue.
	
	upgradeKey := keyValue caseOf: {
		[49 "Taste: 1"]->[#Trap].
		[50 "Taste: 2"]->[#Speedboost].
		[51 "Taste: 3"]->[#Magnet].
		[52 "Taste: 4"]->[#Drain].
		[53 "Taste: 5"]->[#Shockwave]} otherwise: [#Nothing].
	
	upgradeKey ~= #Nothing ifTrue: [
		(self playerBlob upgradeList at: upgradeKey ifAbsent: [SWA014NullUpgrade new]) activateIfNotCooling].
	
	keyValue = 120 ifTrue: [ "Taste: x"
		self delete].	
	keyValue = 112 ifTrue: [ "Taste: p"
		self isPaused 
			ifTrue: [self continue]
			ifFalse: [self pause]].	
! !

!SWA014Game methodsFor: 'input events' stamp: 'TZ 2/2/2017 17:19'!
handlesMouseOver: anEvent
	
	^ true.! !

!SWA014Game methodsFor: 'initialization' stamp: 'MV 2/4/2017 18:22'!
initialize
	super initialize.

	self 
		extent: ActiveWorld extent;
		clipSubmorphs: true;
		
		isPaused: false;
			
		initializeTextures;
		initializeDictionary;
		initializeUpgradesUnavailableToPlayerBlob;
		initializeSpawner;
				
		openInWorld;
		yourself.
		
! !

!SWA014Game methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:10'!
initializeDictionary
	
	self upgradeDictionary: (IdentityDictionary newFrom: {
		#Trap->SWA014TrapUpgrade.
		#Speedboost->SWA014SpeedboostUpgrade.
		#Magnet->SWA014MagnetUpgrade.
		#Drain->SWA014DrainUpgrade.
		#Shockwave->SWA014ShockwaveUpgrade}).! !

!SWA014Game methodsFor: 'initialization' stamp: 'MV 2/5/2017 17:04'!
initializeSpawner

	"The Game doesn't need to know the Spawner, but someone needs to hold a reference on him so he isn't deleted by the garbage collector."
	self spawner: (SWA014Spawner newWith: self).! !

!SWA014Game methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:24'!
initializeTextures
	
	SWA014Trap loadTexture.! !

!SWA014Game methodsFor: 'initialization' stamp: 'TZ 1/27/2017 19:32'!
initializeUpgradesUnavailableToPlayerBlob

	self upgradesUnavailableToPlayerBlob: self upgradeDictionary keys asOrderedCollection.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
isPaused

	^ isPaused.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
isPaused: aBoolean

	isPaused := aBoolean.! !

!SWA014Game methodsFor: 'input events' stamp: 'TZ 2/2/2017 17:19'!
mouseEnter: anEvent

	anEvent hand newKeyboardFocus: self.! !

!SWA014Game methodsFor: 'input events' stamp: 'TZ 2/2/2017 17:19'!
mouseLeave: anEvent

	anEvent hand newKeyboardFocus: nil.! !

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/4/2017 17:19'!
pause

	self playerBlob listOfLivingThings do: [:aLivingThing|
		aLivingThing
			saveSpeed;
			speed: 0].
	self isPaused: true.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/4/2017 14:45'!
playerBlob
	
	^ playerBlob.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/4/2017 14:45'!
playerBlob: aLivingThing
	
	playerBlob := aLivingThing.! !

!SWA014Game methodsFor: 'game events' stamp: 'Tc 2/3/2017 11:32'!
showEvolveOptions
	|evolveMenu|
	
	evolveMenu := SWA014EvolveMenu new.
	evolveMenu
		game: self;
		extent: self extent;
		center: self center;
		showOptions: self upgradesUnavailableToPlayerBlob copy;
		yourself.
		
	self addMorph: evolveMenu.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/5/2017 12:54'!
spawner

	^ spawner.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/5/2017 12:53'!
spawner: aSpawner
	
	spawner := aSpawner.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeBar

	^ upgradeBar.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeBar: aBar

	upgradeBar := aBar.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeDictionary

	^ upgradeDictionary.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeDictionary: aDictionary

	upgradeDictionary := aDictionary.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:50'!
upgradesUnavailableToPlayerBlob

	^ upgradesUnavailableToPlayerBlob.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:49'!
upgradesUnavailableToPlayerBlob: aList

	upgradesUnavailableToPlayerBlob := aList.! !

Morph subclass: #SWA014Menu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Menu commentStamp: 'MV 2/5/2017 17:17' prior: 0!
I am the Menu. You can start or end the game here.
!

!SWA014Menu class methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:48'!
open

	^ SWA014Menu new openInWorld.! !

!SWA014Menu methodsFor: 'initialization' stamp: 'Tc 2/4/2017 12:37'!
initialize 
	super initialize.
	
	self
		extent: World extent;
		addMorph: (ImageMorph new image: (Form fromFileNamed: 'Background.jpg'));
		addMorph: (SWA014StartButton new center: (self width / 2) @ ((12 / 25) * self height));
		addMorph: (SWA014EndButton new center: (self width / 2) @ ((13 / 25) * self height));
		clipSubmorphs: true;
		yourself.! !

Morph subclass: #SWA014UpgradeBar
	instanceVariableNames: 'upgradeCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014UpgradeBar commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am the UpgradeBar. I display, which upgrades you have and how you can use them.
!

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:24'!
add: anUpgrade

	anUpgrade
		center: (self center x) @ (self position y + self offset + (self upgradeCount * self spaceBetweenUpgrades));
		extent: self upgradeSize asPoint;
		yourself.
	
	self 
		increaseUpgradeCount;
		addMorph: anUpgrade;
		yourself.
	! !

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:24'!
increaseUpgradeCount
	
	^ self upgradeCount: self upgradeCount + 1.! !

!SWA014UpgradeBar methodsFor: 'initialization' stamp: 'MV 2/5/2017 13:52'!
initialize
	super initialize.
	
	self 
		extent: 50 @ 500;
		color: Color transparent;
		upgradeCount: 0;
		yourself.! !

!SWA014UpgradeBar methodsFor: 'constants' stamp: 'MV 2/5/2017 16:23'!
offset
	
	^ 50.! !

!SWA014UpgradeBar methodsFor: 'constants' stamp: 'MV 2/5/2017 16:22'!
spaceBetweenUpgrades

	^ 100.! !

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:11'!
upgradeCount

	^ upgradeCount.! !

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:11'!
upgradeCount: aNumber

	upgradeCount := aNumber.! !

!SWA014UpgradeBar methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:12'!
upgradeSize

	^ 75.! !

Morph subclass: #SWA014World
	instanceVariableNames: 'worldSize realCenter image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014World commentStamp: 'MV 2/5/2017 21:02' prior: 0!
I am the World. I am the (next) big thing you can see. In me the game is played. All LivingThings are submorphs of me.!

!SWA014World methodsFor: 'constant' stamp: 'MV 2/5/2017 19:54'!
amountOfPixelsInPattern

	^ 144.! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/3/2017 12:52'!
image

	^ image.! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/3/2017 12:52'!
image: anImage

	image := anImage.! !

!SWA014World methodsFor: 'initialization' stamp: 'MV 2/4/2017 18:11'!
initialize
	super initialize.
		
	self 
		image: (ImageMorph new image: (Form fromFileNamed: 'Pattern.png'));"Be sure, that the picture is in the resource folder with that name"
		addMorph: self image;
		worldSize: 3000;
		extent: self worldSize asPoint;	
		center: 0 asPoint;
		realCenter: 0.0 asPoint;
		yourself.
		! !

!SWA014World methodsFor: 'body transformation' stamp: 'TZ 2/3/2017 12:52'!
radius

	^ worldSize / 2.! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/3/2017 12:52'!
realCenter

	^ realCenter.! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/3/2017 13:35'!
realCenter: aPoint

	realCenter := aPoint.
	self refreshCenter.! !

!SWA014World methodsFor: 'movement' stamp: 'MV 2/5/2017 19:57'!
refreshCenter

	self center: self realCenter.
	self image position: (self position \\ self amountOfPixelsInPattern - self amountOfPixelsInPattern).! !

!SWA014World methodsFor: 'body transformation' stamp: 'TZ 2/3/2017 12:53'!
worldSize

	^ worldSize.! !

!SWA014World methodsFor: 'body transformation' stamp: 'MV 2/4/2017 16:58'!
worldSize: aSize

	worldSize := aSize.! !

ImageMorph subclass: #SWA014Trap
	instanceVariableNames: 'scalingFactor preySpeed listOfLivingThings'
	classVariableNames: 'Texture'
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Trap commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am a Trap. If you have the TrapUpgrade, you can throw me. If i collide with a LivingThing I will stop and shrink it for a short duration.
!

!SWA014Trap class methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:21'!
loadTexture

	Texture := Form fromFileNamed: 'Trap.png'.! !

!SWA014Trap methodsFor: 'constants' stamp: 'TZ 2/2/2017 20:43'!
ailmentDuration

	^ 5000.! !

!SWA014Trap methodsFor: 'collision handling' stamp: 'MV 2/5/2017 13:25'!
checkForCollisionsWithLivingThings

	self listOfLivingThings do: [:aLivingThing|
		(aLivingThing squaredDistanceTo: (aLivingThing calculateRealCenterFor: self center)) < aLivingThing squaredRadius ifTrue: [
			self collideWith: aLivingThing]].! !

!SWA014Trap methodsFor: 'collision handling' stamp: 'TZ 2/2/2017 20:44'!
collideWith: aLivingThing

	self
		inflictAilmentTo: aLivingThing;
		futureDo: #removeAilmentOf: at: self ailmentDuration args: (Array with: aLivingThing);
		delete.! !

!SWA014Trap methodsFor: 'ailments' stamp: 'TZ 2/2/2017 20:38'!
inflictAilmentTo: aLivingThing

	aLivingThing bodysize: aLivingThing bodysize / self scalingFactor.
	aLivingThing saveSpeed.
	aLivingThing speed: 0.! !

!SWA014Trap methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:27'!
initialize
	super initialize.
	
	self 
		image: Texture;
		extent: 15 asPoint;
		scalingFactor: 1.5;
		yourself.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'MV 2/5/2017 13:31'!
listOfLivingThings

	^ listOfLivingThings.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'MV 2/5/2017 13:31'!
listOfLivingThings: aList

	listOfLivingThings := aList.! !

!SWA014Trap methodsFor: 'ailments' stamp: 'TZ 2/2/2017 20:38'!
removeAilmentOf: aLivingThing

	aLivingThing speed: aLivingThing speedSaver.
	aLivingThing bodysize: aLivingThing bodysize * self scalingFactor.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:36'!
scalingFactor

	^ scalingFactor.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:36'!
scalingFactor: aFactor

	scalingFactor := aFactor.! !

!SWA014Trap methodsFor: 'override' stamp: 'TZ 2/2/2017 20:45'!
step

	self checkForCollisionsWithLivingThings.! !

!SWA014Trap methodsFor: 'override' stamp: 'TZ 2/2/2017 20:45'!
stepTime

	^ 100.! !

ImageMorph subclass: #SWA014Upgrade
	instanceVariableNames: 'upgradeOwner coolDownMorph cooling cooldown'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Upgrade commentStamp: 'MV 2/5/2017 20:50' prior: 0!
I am an Upgrade. I am the superclass of all upgrades and implement their basic logic, for example the cooldown.!

SWA014Upgrade subclass: #SWA014DrainUpgrade
	instanceVariableNames: 'drainTarget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014DrainUpgrade commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am a DrainUpgrade. I can drain a LivingThing, so it looses its bodysize, while my upgradeOwner gets size.
!

!SWA014DrainUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 20:57'!
activateUpgrade

	self drainTarget: self upgradeOwner nearestLivingThing.
	self drainTarget borderColor: Color lavender.
	self startStepping.
	self futureDo: #stopStepping at: self drainDuration args: #().
	self drainTarget futureDo: #borderColor: at: 4000 args: (Array with: Color black).! !

!SWA014DrainUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 20:53'!
amountOfDrain

	^ 3.! !

!SWA014DrainUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 20:56'!
drainDuration

	^ 4000.! !

!SWA014DrainUpgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:51'!
drainTarget

	^ drainTarget.! !

!SWA014DrainUpgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:51'!
drainTarget: aLivingThing

	drainTarget := aLivingThing.! !

!SWA014DrainUpgrade methodsFor: 'initialization' stamp: 'TZ 2/2/2017 20:51'!
initialize
	super initialize.
	
	self 
		image: (Form fromFileNamed: 'DrainUpgrade.png');
		stopStepping;
		yourself.! !

!SWA014DrainUpgrade methodsFor: 'override' stamp: 'TZ 2/3/2017 13:36'!
step

	self drainTarget = nil ifFalse: [
		self drainTarget bodysize: (self drainTarget bodysize - self amountOfDrain).
		self upgradeOwner bodysize: (self upgradeOwner bodysize + self amountOfDrain)].! !

!SWA014DrainUpgrade methodsFor: 'override' stamp: 'TZ 2/2/2017 20:55'!
stepTime

	^ 400.
	! !

SWA014Upgrade subclass: #SWA014MagnetUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014MagnetUpgrade commentStamp: 'MV 2/5/2017 20:55' prior: 0!
I am a MagnetUpgrade. I make the LivingThings around my upgradeOwner move towords him.!

!SWA014MagnetUpgrade methodsFor: 'as yet unclassified' stamp: 'TZ 2/3/2017 17:28'!
activateUpgrade

	self startStepping.
	self futureDo: #stopStepping at: self magnetDuration args: #().
! !

!SWA014MagnetUpgrade methodsFor: 'as yet unclassified' stamp: 'TZ 2/3/2017 17:28'!
magnetDuration

	^ 4000.! !

!SWA014MagnetUpgrade methodsFor: 'as yet unclassified' stamp: 'MV 2/4/2017 17:22'!
step
	|force vector|
	self upgradeOwner listOfLivingThings do: [:livingThing|
		(((self upgradeOwner center dist: livingThing center) abs) < 10) ifFalse: [
			force := 225.5 / ((self upgradeOwner squaredDistanceTo: livingThing realCenter)) sqrt.
			vector := livingThing realCenter - self upgradeOwner realCenter.
			vector := vector normalized * force.
			livingThing realCenter: livingThing realCenter - vector.
		]
	].

	self flag: 'TODO: Refactorn und wenn nicht funktionstüchtig removen'.! !

SWA014Upgrade subclass: #SWA014NullUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014NullUpgrade commentStamp: 'MV 2/5/2017 20:56' prior: 0!
I am a NullUpgrade. I intentionally do nothing. So you can use me, if you try to use an upgrade, which you don't got yet.
!

!SWA014NullUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 20:57'!
activateUpgrade
	"This class should do nothing"! !

SWA014Upgrade subclass: #SWA014ShockwaveUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014ShockwaveUpgrade commentStamp: 'MV 2/5/2017 20:57' prior: 0!
I am a ShockwaveUpgrade. I push away all LivingThings, which are nearby me.
!

!SWA014ShockwaveUpgrade methodsFor: 'starting up' stamp: 'MV 2/4/2017 17:23'!
activateUpgrade
	|nearLivingThings|
	
	nearLivingThings := self upgradeOwner listOfLivingThings select: [:livingThing|
		(self upgradeOwner squaredDistanceTo: livingThing realCenter) < (self upgradeOwner radius + self shockRadius) squared]. 
	
	nearLivingThings remove: self upgradeOwner.
	
	nearLivingThings do: [:livingThing|
		livingThing realCenter: livingThing realCenter + (self shockRadius * (self upgradeOwner vectorToPoint: livingThing realCenter))].! !

!SWA014ShockwaveUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 21:04'!
shockRadius
	
	^ 200.! !

SWA014Upgrade subclass: #SWA014SpeedboostUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014SpeedboostUpgrade commentStamp: 'MV 2/5/2017 20:57' prior: 0!
I am a SpeedboostUpgrade. Activate me to get increased speed for a short duration.
!

!SWA014SpeedboostUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 21:12'!
activateUpgrade

	self upgradeOwner 
		speed: self upgradeOwner speed + self speedIncreasement;
		futureDo: #speed: at: self speedboostDuration args: (Array with: (self upgradeOwner speed - self speedIncreasement));
		yourself.! !

!SWA014SpeedboostUpgrade methodsFor: 'initialization' stamp: 'MV 2/5/2017 13:52'!
initialize
	super initialize.
	
	self image: (Form fromFileNamed: 'SpeedBoostUpgrade.png').! !

!SWA014SpeedboostUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 21:10'!
speedIncreasement
	
	^ 11.! !

!SWA014SpeedboostUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 21:09'!
speedboostDuration

	^ 1000.! !

SWA014Upgrade subclass: #SWA014TrapUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014TrapUpgrade commentStamp: 'MV 2/5/2017 20:58' prior: 0!
I am a TrapUpgrade. Activate me to throw a Trap.
!

!SWA014TrapUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 21:12'!
activateUpgrade

 	self spawnTrap.
			! !

!SWA014TrapUpgrade methodsFor: 'initialization' stamp: 'TZ 2/2/2017 21:12'!
initialize
	super initialize.
	
	self image: (Form fromFileNamed: 'TrapUpgrade.png').! !

!SWA014TrapUpgrade methodsFor: 'world manipulation' stamp: 'MV 2/5/2017 13:34'!
spawnTrap
	|aTrap|
	
	aTrap := SWA014Trap new.
	
	aTrap
		center: self upgradeOwner center - (self upgradeOwner direction * self upgradeOwner bodysize);
		listOfLivingThings: self upgradeOwner listOfLivingThings;
		yourself.
	
	self upgradeOwner swaworld addMorph: aTrap.! !

!SWA014Upgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 20:49'!
activateIfNotCooling

	self cooling ifFalse: [
		self
			activateUpgrade;
			cooling: true;
			futureDo: #cooling: at: (self cooldown) args: (Array with: false);
			yourself].
	
	self coolDownMorph
		show;
		futureDo: #hide at: self cooldown args: #();
		yourself.! !

!SWA014Upgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 20:50'!
activateUpgrade
	"The different Upgrades should overwrite this. This method gets called, when the player wants to use an Upgrade."
	"To add a new Upgrade change: Game->initializeDictionary, Game->handleKeystroke, and right now Game->showEvolveOptions."
	
	self SubclassResponsibility.
	! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
coolDownMorph

	^ coolDownMorph.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
coolDownMorph: aMorph

	coolDownMorph := aMorph.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
cooldown

	^ cooldown.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
cooldown: aTime

	cooldown := aTime.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
cooling

	^ cooling.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
cooling: aBoolean

	cooling := aBoolean.! !

!SWA014Upgrade methodsFor: 'initialization' stamp: 'TZ 2/2/2017 20:48'!
initialize
	super initialize.
	
	self
		coolDownMorph: ImageMorph new; 
		cooling: false;
		cooldown: 10000;
		addMorph: self coolDownMorph;
		yourself.
		
	self coolDownMorph
		image: (Form fromFileNamed: 'Cooldown.png');
		hide;
		yourself.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
upgradeOwner

	^ upgradeOwner.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
upgradeOwner: aLivingThing

	upgradeOwner := aLivingThing.! !

SimpleButtonMorph subclass: #SWA014ChooseButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014ChooseButton commentStamp: 'MV 2/5/2017 17:06' prior: 0!
I am a ChooseButton. I will be shown in an EvolveOptionWindow and if you click on me, you will get the Upgrade, which is specified above me. !

!SWA014ChooseButton methodsFor: 'initialization' stamp: 'Tc 2/3/2017 11:25'!
initialize 
	super initialize.

	self 
		label: 'Choose!!';
		color: Color red;
		yourself.! !

!SWA014ChooseButton methodsFor: 'event handling' stamp: 'TZ 1/27/2017 16:31'!
mouseDown: anEvent

	"The owner is always an EvolveOption"
	self owner getChosen.! !

SimpleButtonMorph subclass: #SWA014EndButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014EndButton commentStamp: 'Tc 2/5/2017 14:17' prior: 0!
I am the EndButton in the Menu. If you click on me, you will end the game.!

!SWA014EndButton methodsFor: 'initialization' stamp: 'Tc 2/3/2017 11:25'!
initialize 
	super initialize.

	self 
		label: 'End Game';
		color: Color red;
		yourself.! !

!SWA014EndButton methodsFor: 'event handling' stamp: 'TZ 1/27/2017 16:31'!
mouseDown: anEvent

	"The owner is the menu, this button closes it"
	self owner delete.! !

SimpleButtonMorph subclass: #SWA014StartButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014StartButton commentStamp: 'MV 2/5/2017 17:36' prior: 0!
I am the StartButton. If you click on me I will start the game.!

!SWA014StartButton methodsFor: 'initialization' stamp: 'TZ 1/27/2017 16:19'!
initialize 
	super initialize.

	self 
		label: 'Start Game';
		color: Color green;
		yourself.! !

!SWA014StartButton methodsFor: 'event handling' stamp: 'TZ 2/2/2017 20:34'!
mouseDown: anEvent

	SWA014Game new.! !

!Random methodsFor: '*SWA014' stamp: 'TZ 2/3/2017 15:58'!
isTrueIn: aNumber outOf: aNumberOfCases
	
	^ (self nextInt: aNumberOfCases) <= aNumber.! !

StringMorph subclass: #SWA014Score
	instanceVariableNames: 'scoreValue'
	classVariableNames: 'Highscore'
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Score commentStamp: 'MV 2/5/2017 21:08' prior: 0!
I am the Score. I will show you, how many points you got so far and if you are good enough, you will beat the highscore, which I hold. I observe the PlayerBlob to give you your points when you eat an Enemy.
!

!SWA014Score class methodsFor: 'accessing' stamp: 'TZ 2/3/2017 18:12'!
highscore
	
	 ^ Highscore.! !

!SWA014Score class methodsFor: 'accessing' stamp: 'TZ 2/3/2017 18:10'!
highscore: aScore
	
	Highscore := aScore.! !

!SWA014Score methodsFor: 'updating' stamp: 'TZ 2/3/2017 18:18'!
increaseScore
	
	self scoreValue: self scoreValue + 10.
	self acceptValue: self scoreValue.
	self color: self randomColor.
	self updateHighscore.
	! !

!SWA014Score methodsFor: 'initialization' stamp: 'TZ 2/3/2017 18:15'!
initialize 
	super initialize.
	
	self
		scoreValue: 0;
		acceptValue: 0;
		color: Color red;
		width: 500;
		height: 500;
		fontName: 'BitstreamVeraSans' size: 60;
		initializeHighscoreIfNecessary;
		yourself.! !

!SWA014Score methodsFor: 'initialization' stamp: 'TZ 2/3/2017 18:15'!
initializeHighscoreIfNecessary

	SWA014Score highscore = nil ifTrue: [SWA014Score highscore: 0].! !

!SWA014Score methodsFor: 'accessing' stamp: 'TZ 2/3/2017 18:17'!
randomColor
	
	^ {Color green. Color red. Color yellow. Color blue. Color orange. Color lavender. Color black} atRandom.! !

!SWA014Score methodsFor: 'accessing' stamp: 'MV 2/5/2017 17:21'!
scoreValue

	^ scoreValue.! !

!SWA014Score methodsFor: 'accessing' stamp: 'MV 2/5/2017 17:21'!
scoreValue: aValue

	scoreValue := aValue.! !

!SWA014Score methodsFor: 'updating' stamp: 'TZ 2/3/2017 17:50'!
update: aMessage
	
	self increaseScore.! !

!SWA014Score methodsFor: 'updating' stamp: 'TZ 2/3/2017 18:21'!
updateHighscore
	
	SWA014Score highscore: (SWA014Score highscore max: self scoreValue). ! !

!Point methodsFor: '*SWA014' stamp: 'MV 2/4/2017 18:37'!
normalizedWithoutZeroDivide
	^ self = 0 asPoint
		ifTrue: [0]
		ifFalse: [self normalized].
	! !
